/**********************************************************************
Practicas SITR: programacion de sockets

Ejemplo de programacin sobre sockets PF_INET de tipo STREAM (TCP)
Extremo Cliente  ------>

Compilar con: 		gcc -o tcpclt1 tcpclt1.c -lnsl  -lsocket -lposix4
Compilar linux: 	gcc -o tcpclt1 tcpclt1.c -lnsl  -lrt

	
Uso del programa (2):
	tcpclt1		ip_servidor		puerto_servidor 
	tcpclt1		ip_servidor		puerto_servidor		"mensaje"
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>


#define DIM	1024		// tamao buffer de datos de mensaje

// Mensaje a enviar si no se indica como parmetro
char *mensaje_def= "Este es el mensaje enviado desde el cliente SOCK_STREAM";


int main(int argc, char *argv[])
{
	int socketID;					// identificador de dispositivo para el socket
	struct sockaddr_in datosSocketSrv;	// direccin IP, tipo y puerto del servidor
	struct sockaddr_in datosSocketClt;	// direccin IP, tipo y puerto del cliente
	struct hostent *hp;				// direccin IP a partir de nombre simblico
	ssize_t res;					// resultado de un sento()
	size_t longdir;					// tamao estructura socaddr_in

	char mensaje[DIM];				// buffer memoria para los mensajes

	
	// Comprueba los parmetros
	//****************************************************************************
	if(argc<3)
	{
		printf("Uso del programa: tcpclt1   servidor  puerto  <mensaje>\n");
		exit(0);
	}	
	if(argc==4)		strcpy(mensaje,argv[3]);		// mensaje pasado como parmetro
	else			strcpy(mensaje,mensaje_def);	// mensaje por defecto


/* 
1 etapa:  Creamos un socket PF_INET de tipo SOCK_STREAM (obtenemos un manejador)
***********************************************************************
*/
	socketID = socket( PF_INET, SOCK_STREAM, 0);
	if(socketID<0)
	{ 
		perror("Creando socket");
		exit(-1);
	}
	
/* 
2 etapa: Obtenemos la direccin IP y puerto del servidor a partir de los
parmetros
***********************************************************************
*/
	// convierte el nombre simblico a una direccin IP
	hp=gethostbyname(argv[1]);
	if(hp == 0)
	{ 
		fprintf(stderr, "\n %s: host desconocido\n", argv[1]);
		close(socketID);
		exit(-2);
	}
	// copiamos la direccin IP en la estructura datosSocket
	memcpy( &(datosSocketSrv.sin_addr),  hp->h_addr, hp->h_length);
	datosSocketSrv.sin_family=AF_INET; // tipo de direccin
	// convierte el puerto a un nmero de red (htons)
	datosSocketSrv.sin_port = htons(atoi(argv[2]));

/* 
3 etapa:  Establecemos la conexin
************************************************************************/
	res = connect(socketID, (struct sockaddr *) &datosSocketSrv, sizeof datosSocketSrv);
	if(res == -1)
	{
		perror("Conexion no aceptada");
		close(socketID);
		exit(-3);
	}
	// Datos del puerto del cliente (asignados automticamente tras la conexin)
	longdir=sizeof datosSocketClt;
	res = getsockname(socketID,  (struct sockaddr *) &datosSocketClt, &longdir);
	if(res==0)		
		printf("---Cliente -> IP: %s,  Puerto: %u---\n\n", 
			inet_ntoa(datosSocketClt.sin_addr), 	ntohs(datosSocketClt.sin_port));	

/*
4 etapa:  Enviamos informacin al servidor. Opciones:
ssize_t send(int socket, const void *buffer,  size_t length, int flags);
ssize_t write(int fildes, const void *buf, size_t nbytes)
***********************************************************************
*/
	res = send(socketID, mensaje, strlen(mensaje)+1, 0);
	if(res>0)
	{	// muestra lo que ha enviado
		printf("%d bytes Enviados a [%s:%u]: %s\n", res, 
				inet_ntoa(datosSocketSrv.sin_addr), ntohs(datosSocketSrv.sin_port), mensaje);
	}

/*
5 etapa:   Cuando terminemos de trasmitir datos Cerrar el socket
**********************************************************************
*/
	close(socketID);
	printf("[Conexion Terminada]\n"); 

	exit(0);
}
