/**********************************************************************
Practicas SITR: programacion de sockets

Ejemplo de programacion sobre sockets PF_INET de tipo STREAM (TCP)
------> Extremo Servidor 

Compilar con: 		gcc -o tcpsrv1 tcpsrv1.c -lnsl -lsocket  -lposix4    	
Compilar linux:		gcc -o tcpsrv1 tcpsrv1.c -lnsl -lrt	

Uso del programa (2):   
		tcpsrv1				-> usa puerto por defecto (se muestra en la consola)
		tcpsrv1 puerto
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pthread.h>

#define DIM			1024	// tamao buffer mensaje
#define CONEXIONES	5		// nmero de trasnmisiones esperadas
#define PUERTO		0		// asigna automticamente un puerto libre


int main(int argc, char *argv[])
{
	int socketID;					// identificador de dispositivo para el socket
	int socketDialogo;				// identificador socket dialogo
	in_port_t puerto;				// puerto publico
	struct sockaddr_in datosSocketSrv;	// direccin IP, tipo y puerto del servidor
	struct sockaddr_in datosSocketClt;	// direccin IP, tipo y puerto del cliente
	struct hostent *hp;				// direccin IP a partir de nombre simblico
	ssize_t res;					// resultado de un sento()
	size_t longdir;					// tamao estructura socaddr_in

	char mensaje[DIM];				// buffer memoria para los mensajes
	int cont;						// contador de conexiones

	// Comprueba los parmetros
	//****************************************************************************
	if(argc>2)
	{
		printf("Uso del programa: tcpsrv1  <puerto>\n");
		exit(0);
	}		
	if(argc==2)		puerto=atoi(argv[1]);	// puerto pasado como parmetro
	else			puerto=PUERTO;			// puerto por defecto

	
/* 
1 etapa:
Creamos un socket PF_INET de tipo SOCK_STREAM (obtenemos un manejador)
***********************************************************************
*/
	socketID = socket( PF_INET, SOCK_STREAM, 0);
	if(socketID<0)
	{ 
		perror("Creando socket"); exit(-1);
	}

/* 
2 etapa:
Configuramos la estructura direccin:
puerto -> IPPORT_RESERVED+XX seleccin fija
la direccin IP se toma la suya propia INADDR_ANY ya que es el servidor
***********************************************************************
*/
	datosSocketSrv.sin_family=AF_INET;			// tipo de direccin
	datosSocketSrv.sin_port = puerto;			// convierte el puerto a un nmero de red (htons)
	datosSocketSrv.sin_addr.s_addr = htonl(INADDR_ANY); // convierte la direccin IP a un nmero de red (htonl)

/*
 3 etapa:
Nombramos el socket asignndole el puerto y direccin
***********************************************************************
*/
	res = bind(socketID, (struct sockaddr *) &datosSocketSrv, sizeof datosSocketSrv);
	if(res<0) 
	{
		perror("Nombrando el socket");
		close(socketID);
		exit(-2);
	}

	// Pedimos los datos del puerto asignado
	longdir=sizeof datosSocketSrv;
	res = getsockname(socketID, (struct sockaddr *) &datosSocketSrv, &longdir);
	if(res<0) 
	{
		perror("Leyendo informacion  del socket");
		close(socketID);
		exit(-2);
	}
	// muestra los datos del servidor
	printf("---Servidor IP: %s,  Puerto: %u---\n", 
		inet_ntoa(datosSocketSrv.sin_addr),ntohs(datosSocketSrv.sin_port));	

/*
4 etapa:
Poner en escucha el servidor con una cola de 5 mensajes
cuando aperece una conexin aparece un nuevo socket
***********************************************************************
*/
	listen(socketID, CONEXIONES);
	// aceptar hasta 5 conexiones
	for(cont=0; cont<CONEXIONES; cont++)
	{
		printf("\nEsperando Conexion (%d) ....\n", cont+1);

		longdir = sizeof datosSocketClt;
		socketDialogo = accept(socketID, (struct sockaddr *) &datosSocketClt, &longdir);
		
		// se ha creado un nuevo socket para la conexin en curso
		if(socketDialogo <0)
		{
			perror("Conexion no aceptada");
			continue;
		}
		/*
		5 etapa:   Recibimos informacin del cliente. Opciones:
		ssize_t read(int fildes, void *buf, size_t nbytes)
		ssize_t recv(int socket, void *buffer, size_t length, int flags);
		***********************************************************************
		*/
		res = recv(socketDialogo, mensaje, DIM, 0);
		if(res>0)
		{
			// muestra lo que ha leido
			printf("[Conexion %d, Recibido (%d bytes) desde %s:%u <-] %s\n", cont+1, res, 
						inet_ntoa(datosSocketClt.sin_addr), ntohs(datosSocketClt.sin_port), mensaje);		
		}
		close(socketDialogo); // cierra la conexin: socket de dialogo
		printf("[Conexion %d Terminada]\n", cont+1); 
	}
/*
6 etapa:
Cerrar el socket de Escucha
**********************************************************************
*/
	close(socketID);
	printf("[Servidor Terminado]\n"); 
	exit(0);
}
