/*
  *********************************************************************
  
  Ejemplo de programacion sobre sockets PF_INET de tipo Datagrama (UDP)
  
  Extremo Cliente ------>
  
Compilar con: 		gcc -o udpclt1 udpclt1.c -lnsl -lsocket  -lposix4 
Compilar linux: 	gcc -o udpclt1 udpclt1.c -lnsl -lrt
	
Uso del programa (2):
	udpclt1		ip_servidor		puerto_servidor 
	udpclt1		ip_servidor		puerto_servidor		"mensaje"

  *********************************************************************
*/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>


#define DIM	1024		// tamao buffer de datos de mensaje

// Mensaje a enviar si no se indica como parmetro
char *mensaje_def= "Este es el mensaje enviado desde el cliente SOCK_DGRAM";

int main(int argc, char *argv[])
{
	int socketID; // identificador de dispositivo para el socket
	struct sockaddr_in datosSocketSrv;	// direccin IP, tipo y puerto del servidor
	struct sockaddr_in datosSocketClt;	// direccin IP, tipo y puerto del cliente
	struct hostent *hp;				// direccin IP a partir de nombre simblico
	ssize_t res;					// resultado de un sento()
	size_t longdir;					// tamao estructura socaddr_in

	char mensaje[DIM];				// buffer memoria para los mensajes


	// Comprueba los parmetros
	//****************************************************************************
	if(argc<3)
	{
		printf("Uso del programa: udpclt1   servidor  puerto  <mensaje>\n");
		exit(0);
	}	
	if(argc==4)		strcpy(mensaje,argv[3]);		// mensaje pasado como parmetro
	else			strcpy(mensaje,mensaje_def);	// mensaje por defecto

	/*
	  1 etapa: Creamos un socket PF_INET de tipo SOCK_DGRAM (obtenemos un manejador)
	*********************************************************************************
	*/
	socketID = socket( PF_INET, SOCK_DGRAM, 0);
	if(socketID<0)
	{
		perror("Creando socket");
		exit(-1);
	}


	/*
	 2 etapa: Obtenemos la direccin IP y puerto del servidor 
				a partir de los	 parmetros
	***********************************************************************
	*/

	// convierte el nombre simblico a una direecin IP
	hp=gethostbyname(argv[1]);
	if(hp == NULL)
	{
		fprintf(stderr, "\n %s: host desconocido\n", argv[1]);
		close(socketID);
		exit(-2);
	}
	// copiamos la direccin IP en la estructura datosSocket
	memcpy( &(datosSocketSrv.sin_addr), hp->h_addr, hp->h_length);
	datosSocketSrv.sin_family=AF_INET;					// tipo de direccin
	datosSocketSrv.sin_port = htons(atoi(argv[2]));	// convierte el puerto a un nmero de red (htons)
	

	/*
	 3 etapa:	 Enviamos inforamcin al servidor:
	 	ssize_t sendto(int socket, const void *message,	size_t length, int flags,
          		const struct sockaddr *dest_addr, size_t dest_len);
	***********************************************************************
	*/
	res = sendto(socketID, mensaje, strlen(mensaje)+1, 0,
			(struct sockaddr *) &datosSocketSrv, sizeof datosSocketSrv);
	if(res>0)
	{	// muestra lo que ha enviado
		printf("%d bytes Enviados a [%s:%u]: %s\n", res, 
				inet_ntoa(datosSocketSrv.sin_addr), ntohs(datosSocketSrv.sin_port), mensaje);

		// Datos del puerto del cliente (asignados automticamente tras la primera transmisin
		longdir=sizeof datosSocketClt;
		res = getsockname(socketID,  (struct sockaddr *) &datosSocketClt, &longdir);
		if(res==0)		
			printf("---Cliente -> IP: %s,  Puerto: %u---\n\n", 
				inet_ntoa(datosSocketClt.sin_addr), 	ntohs(datosSocketClt.sin_port));	

	}

	/*
	 4 etapa:
	 Cerrar el socket
	 **********************************************************************
	*/		
	close(socketID);
	
	exit(0);	

}
