/*
  *********************************************************************
  
  Ejemplo de programacion sobre sockets PF_INET de tipo Datagrama (UDP)
  
   ------> Extremo SERVIDOR 
  
Compilar con: 		gcc -o udpsrv1 udpsrv1.c -lnsl -lsocket   -lposix4     	
Compilar linux: 	gcc -o udpsrv1 udpsrv1.c -lnsl -lrt    	

Uso del programa (2):   
		udpsrv1				-> usa puerto por defecto (se muestra en la consola)
		udpsrv1 puerto

  *********************************************************************
*/
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>


#define DIM			1024	// tamao buffer mensaje
#define CONEXIONES	2		// nmero de trasnmisiones esperadas
#define PUERTO		0		// asigna automticamente un puerto libre

int main(int argc, char *argv[])
{
	int socketID;						// identificador de dispositivo para el socket
	in_port_t puerto;						// puerto publico
	struct sockaddr_in datosSocketSrv;		// direccin IP, tipo y puerto swl Servidor
	struct sockaddr_in datosSocketClt;		// direccin IP, tipo y puerto del cliente
	ssize_t res;							// resultado de un recvfrom()
	size_t longdir;							// tamao estructura  sockaddr_in

	char mensaje[DIM];						// buffer memoria para los mensajes
	int cont;								// contador de conexiones

	
	// Comprueba los parmetros
	//****************************************************************************
	if(argc>2)
	{
		printf("Uso del programa: udpsrv1  <puerto>\n");
		exit(0);
	}	
	
	if(argc==2)		puerto=atoi(argv[1]);	// puerto pasado como parmetro
	else			puerto=PUERTO;			// puerto por defecto

	/*
	  1 etapa: Creamos un socket PF_INET de tipo SOCK_DGRAM (obtenemos un manejador)
	***********************************************************************
	*/
	socketID = socket( PF_INET, SOCK_DGRAM, 0);
	if(socketID<0)
	{
		perror("Creando socket");
		exit(-1);
	}


	/*
	 2 etapa:  Configuramos la estructura direccin: 
	 		puerto 0 -> primero libre
	 la direccin IP se toma la suya propia INADDR_ANY ya que es el servidor
	***********************************************************************
	*/

	datosSocketSrv.sin_family=AF_INET;			// tipo de direccin
	datosSocketSrv.sin_port = puerto;			// convierte el puerto a un nmero de red (htons)
	datosSocketSrv.sin_addr.s_addr = htonl(INADDR_ANY); // convierte la direccin IP a un nmero de red (htonl)


	/*
	 3 etapa: Nombramos el socket asignndole el puerto y direccin
	***********************************************************************
	*/
	res = bind(socketID, (struct sockaddr *) &datosSocketSrv, sizeof datosSocketSrv);
	if(res<0) 
	{
		perror("Nombrando el socket");
		close(socketID);
		exit(-2);
	}
	
	// Pedimos los datos del puerto asignado
	longdir=sizeof datosSocketSrv;
	res = getsockname(socketID, (struct sockaddr *) &datosSocketSrv, &longdir);
	if(res<0) 
	{
		perror("Leyendo informacion  del socket");
		close(socketID);
		exit(-2);
	}
	
	// muestra los datos del servidor
	printf("---Servidor IP: %s,  Puerto: %u---\n", 
		inet_ntoa(datosSocketSrv.sin_addr),ntohs(datosSocketSrv.sin_port));	
	
	/*
	 4 etapa:	 Recibimos inforamcin del cliente. Opciones:
	 	ssize_t recvfrom(int socket, void *message, size_t length, int flags,
          		const struct sockaddr *dest_addr, size_t *dest_len);
	***********************************************************************
	*/
	for(cont=0; cont<CONEXIONES; cont++)
	{
		printf("\nEsperando datos....\n");
		longdir=sizeof datosSocketClt;
		res = recvfrom(socketID, mensaje, DIM, 0,
						(struct sockaddr *) &datosSocketClt, &longdir);
		if(res>0) 
		{	// muestra los datos del cliente y los datos recibidos
			printf("%d bytes Recibidos desde [%s:%u]: %s\n", res, 
					inet_ntoa(datosSocketClt.sin_addr), ntohs(datosSocketClt.sin_port), mensaje);
		}
	}

	/*
	 5 etapa:	 Cerrar el socket
	 **********************************************************************
	*/		
	close(socketID);
	
	exit(0);		
}
